# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import papyon

from gl_config import gl_config, INCOMING_MSG, OUTGOING_MSG, INFO_MSG, \
    OUTGOING_OFFLINE_MSG, INCOMING_OFFLINE_MSG
from gl_fct import format_nick
from gui.chatnotebook import ChatNotebook
from gui.info_box import InfoBox
from gui.chatview import ChatView

class ConversationEvents(papyon.event.ConversationEventInterface):

    def on_conversation_user_joined(self, contact):
        if contact not in self.chatinstance.users:
            self.chatinstance.users.add(contact)
            text = '%s a rejoint la conversation' % contact.display_name
            self.chatinstance.chatview.add_entry_content(INFO_MSG, text)
            self.chatinstance.info_box.new_participant(contact)
            self.chatinstance.update_title()

    def on_conversation_user_left(self, contact):
        users = self.chatinstance.users
        if len(users) > 1:
            text = '%s a quitté la conversation' % contact.display_name
            self.chatinstance.chatview.add_entry_content(INFO_MSG, text)
            self.chatinstance.info_box.remove_participant(contact)
            self.chatinstance.update_title()
            users.remove(contact)

    def on_conversation_oim_message_received(self, sender, message):
        ''' This method is not called from papyon '''
        self.on_conversation_message_received(sender, message,
                INCOMING_OFFLINE_MSG)

    def on_conversation_message_received(self, sender, msg, fmt=INCOMING_MSG):
        if sender.memberships & papyon.Membership.BLOCK:
            return # contact is blocked, ignore his/her messages

        if fmt == INCOMING_MSG:
            msg = msg.content
        chat = self.chatinstance
        chat.chatview.add_entry_content(fmt, msg, sender.display_name)
        nb = gl_config.notebooks
        if nb.page_num(chat) != nb.get_current_page():
            chat.set_highlight_nb()

    def __init__(self, conv, chatinstance):
        self.chatinstance = chatinstance
        papyon.event.ConversationEventInterface.__init__(self, conv)

class ChatInstance(ChatNotebook):

    ''' Represent a conversation with one or more contact '''

    def send_msg(self, widget, text):

        ''' User outgoing message '''

        if not text:
            return False
        if len(self.users) == 1:
            user = list(self.users)[0]
            if not user.connected:
                gl_config.client.oim_box.send_message(user, text)
                self.chatview.add_entry_content(OUTGOING_OFFLINE_MSG, text)
                return True
            else:
                self.conv.invite_user(user)
        msg = papyon.ConversationMessage(text)
        self.conv.send_text_message(msg)
        self.chatview.add_entry_content(OUTGOING_MSG, text)
        return True

    def update_title(self):
        if len(self.conv.participants) > 1:
            self.title = 'Conversation groupée'
        else:
            for contact in self.users:
                self.title = format_nick(contact.display_name)
                break

    def __init__(self, client, participants, conv=None):

        ''' If the conversation does not exists (conv=None), it comes
            from a roaster clic, so we create it. If it exists, we need
            to rattach it. '''

        self._title = ''
        self.client = client
        self.users = set() # Users that shoud be in the conversation : we
                           # can not rely on the participants set in the
                           # lib (users are not present when we just open
                           # the chat window, we need to wait them to join),
                           # conversation can be closed to inactivity, …

        self.info_box = info_box = InfoBox(participants)
        self.chatview = chatview = ChatView()
        ChatNotebook.__init__(self, gl_config.notebooks, info_box, chatview)
        self.show_all()

        if conv == None:
            self.conv = conv = papyon.Conversation(self.client, participants)
            self.title = format_nick(participants[0].display_name)
            self.users.add(participants[0]) # There is just one in that case
            gl_config.notebooks.append_chat(self)
            gl_config.notebooks.set_current_page(self.tab)
        else:
            self.conv = conv
            self.users = participants.copy() # We received the original set
            gl_config.notebooks.append_chat(self)

        self.conv_events = ConversationEvents(conv, self)

        self.close_button.connect('clicked', gl_config.chats.close_conv, self)
        chatview.entry.connect('msg_sent', self.send_msg)

        self.update_title()

    def title():
        def fget(self):
            return self._title
        def fset(self, title):
            self._title = title
            self.set_label(title)
        return locals()
    title = property(**title())

    @property
    def tab(self):
        return gl_config.notebooks.page_num(self)
