# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import papyon

from gl_config import gl_config

from chat_instance import ChatInstance
from gui.notebooks import Notebooks

class OfflineChat(papyon.event.OfflineMessagesEventInterface):

    def on_oim_state_changed(self, state):
        print 'oim state changed', state

    def on_oim_messages_received(self, messages):
        print 'oim messages received, fetch them…'
        self.__client.oim_box.fetch_messages()

    def on_oim_messages_fetched(self, messages):
        for message in sorted(messages):
            sender = message.sender
            chat = gl_config.chats._check_if_opened(sender)
            if not chat:
                chat = gl_config.chats.new_instance([sender], None)
            meth = chat.conv_events.on_conversation_oim_message_received
            meth(sender, str(message))
        self.__client.oim_box.delete_messages()

    def on_oim_messages_deleted(self):
        print 'oim messages deleted'

    def on_oim_message_sent(self, recipient, message):
        print 'oim message sent', recipient, message

    def __init__(self, client):
        self.__client = client
        papyon.event.OfflineMessagesEventInterface.__init__(self, client)

class Chats(papyon.event.InviteEventInterface):

    ''' Main object which grab roaster clic or incoming conversation,
        and store all chat instances '''

    def close_conv(self, widget, chat_instance):
        chat_instance.conv.leave()
        self.notebooks.close_tab(chat_instance.tab)
        self.instances.remove(chat_instance)

    def _check_if_opened(self, contact):

        ''' Grab tab notebook focus if already open '''

        for chat in self.instances:
            if contact in chat.users:
                return chat
        return None

    def new_instance(self, participants, conv=None):
        new_chat = ChatInstance(self.__client, participants, conv)
        self.instances.append(new_chat)
        return new_chat

    def engage_conv(self, treeview, user_param=None):

        ''' Engage conversation with one contact (from roaster clic) '''

        # Which contact ?
        treeviewmodel = treeview.get_model()
        (path, column_focus) = treeview.get_cursor()
        selected_iter = treeviewmodel.get_iter(path)
        contact = treeviewmodel.get_value(selected_iter, 0)

        # Ignore Group clic
        if not isinstance(contact, papyon.Contact):
            return

        chat = self._check_if_opened(contact)
        if chat:
            self.notebooks.set_current_page(chat.tab)
            chat.chatview.grab_entry_focus()
            return

        # Not opened, create new one
        new_chat = self.new_instance([contact], conv=None)
        new_chat.chatview.grab_entry_focus()

    def on_invite_conversation(self, conv):

        ''' Incoming conversation invitation '''

        if len(conv.participants) == 1:
            contact = list(conv.participants)[0]
            if self._check_if_opened(contact):
                return
        self.new_instance(conv.participants, conv)

    def __init__(self, client):

        self.__client = client
        self.instances = [] # Array of chat instances
        self.notebooks = Notebooks()

        papyon.event.InviteEventInterface.__init__(self, client)
        self.__offline_handle = OfflineChat(client)
