# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import papyon

# Files management (display pictures)
from os import path
import md5
# ---

from gl_config import gl_config, DP_CACHE

class ContactEvents(papyon.event.ContactEventInterface):

    ''' Catch contact events and update the contact list as well '''

    def _msn_object_retrieved(self, msn_object, contact):

        ''' MSN Object is retrieved, we save the file and
            update info boxes where avatars are displayed '''

        print(' |--- MSN Object retrieved from %s' % contact.account)
        if msn_object._type == papyon.p2p.MSNObjectType.DISPLAY_PICTURE:
            filename = md5.new(contact.account).hexdigest()
            filepath = path.join(DP_CACHE, filename)
            avatar = open(filepath, 'w+b')
            avatar.write(msn_object._data.read())
            avatar.close()
            contact.set_dp(filepath)
            for chat in gl_config.chats.instances:
                contact_box = chat.info_box.get_participant_box(contact)
                if contact_box != None:
                    contact_box.update_avatar()

    def on_contact_msn_object_changed(self, contact):
        if contact.account == self.__client.profile.account:
            return
        if contact.msn_object:
            self.__client.msn_object_store.request(contact.msn_object,
                    (self._msn_object_retrieved, contact))

    def on_contact_memberships_changed(self, contact):
        for chat in gl_config.chats.instances:
            contact_box = chat.info_box.get_participant_box(contact)
            if contact_box is not None:
                contact_box.update_warning_label()
        self.clist.update_contact(contact)

    def on_contact_presence_changed(self, contact):
        self.clist.update_contact(contact)
        for chat in gl_config.chats.instances:
            contact_box = chat.info_box.get_participant_box(contact)
            if contact_box != None:
                contact_box.update_display_presence()

    def on_contact_display_name_changed(self, contact):
        self.clist.update_contact(contact)
        for chat in gl_config.chats.instances:
            contact_box = chat.info_box.get_participant_box(contact)
            if contact_box != None:
                contact_box.update_display_name()
                chat.update_title()

    def on_contact_personal_message_changed(self, contact):
        for chat in gl_config.chats.instances:
            contact_box = chat.info_box.get_participant_box(contact)
            if contact_box != None:
                contact_box.update_display_personal_message_changed()

    def __init__(self, client, clist):
        self.__client = client
        self.clist = clist
        papyon.event.ContactEventInterface.__init__(self, client)
