# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import papyon
import gtk
import gobject
from gl_config import gl_config

from gui.incoming_add_contact_win import IncomingAddContactWin

class Group(object):

    def __init__(self, title, pixpath):
        self.str = title
        self.pic = gtk.Image()
        self.pic.set_from_file(pixpath)
        self.pixbuf = self.pic.get_pixbuf()

class ContactList(gtk.TreeStore):

    __gsignals__ = \
    {
        'add_contact' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (str,))
    }

    ONLINE_STR = 'Mes contacts connectés :'
    OFFLINE_STR = 'Mes contacts déconnectés :'

    def add_contact(self, widget, addr):

        ''' Add contact in the address book '''
        self.client.address_book.add_messenger_contact(addr)

    def rm_contact(self, widget, contact):

        ''' Remove contact from the clist '''
        self.client.address_book.delete_contact(contact)

    def block_contact(self, widget, contact):

        ''' Block contact in the list '''
        self.client.address_book.block_contact(contact)

    def unblock_contact(self, widget, contact):

        ''' Unblock contact in the list '''
        self.client.address_book.unblock_contact(contact)

    def insert_contact(self, contact):

        ''' Insert contact in the treeview/store '''
        grp = self._online if contact.connected else self._offline
        row_values = (contact, contact.clist_str, contact.status_pic)
        contact.treeiter = self.append(parent=grp, row=row_values)
        if contact.memberships & papyon.Membership.PENDING \
                and contact.account not in gl_config.already_accepted:  # hack, contact is inserted
                                                                        # multiple times when accepted, ask only ONCE
            gobject.idle_add(self.incoming_contact, contact)

    def incoming_contact(self, contact):
        win = IncomingAddContactWin(None, contact)
        response = win.run()
        win.destroy()
        if response == 1:    # we accepted the contact
            self.client.address_book.accept_contact_invitation(contact)
            gl_config.already_accepted.append(contact.account)
            self.insert_contact(contact)
        else:                   # refuse the contact
            self.client.address_book.decline_contact_invitation(contact)
        win.destroy()

    def remove_contact(self, contact):

        ''' Remove contact from the treeview/store '''
        self.remove(contact.treeiter)

    def update_contact(self, contact):
        self.set_value(contact.treeiter, 1, contact.clist_str)
        self.set_value(contact.treeiter, 2, contact.status_pic)
        iter_bak = contact.treeiter
        self.insert_contact(contact)
        self.remove(iter_bak)

    def __init__(self, client):

        self.client = client

        gtk.TreeStore.__init__(self, object, str, gtk.gdk.Pixbuf)

        # Parent entry (online/offline branches)
        online_grp = Group(ContactList.ONLINE_STR, 'images/status/online.png')
        offline_grp = Group(ContactList.OFFLINE_STR, 'images/status/offline.png')
        self._online = self.append(parent=None,
                row=(online_grp, online_grp.str, online_grp.pixbuf))
        self._offline = self.append(parent=None,
                row=(offline_grp, offline_grp.str, offline_grp.pixbuf))

        # Fill contact list with all contacts (offline and online)
        for contact in self.client.address_book.contacts:
            self.insert_contact(contact)
