# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import papyon

import gobject

from ConfigParser import ConfigParser
from shutil import copy
from os import getenv, mkdir, path
from sys import exit as sysexit

INCOMING_MSG, \
OUTGOING_MSG, \
INCOMING_OFFLINE_MSG, \
OUTGOING_OFFLINE_MSG, \
INFO_MSG, \
ERR_MSN = range(6)

WIN_TITLE = 'oMSN - Ordissimo MSN'
MSN_SERVER = 'messenger.hotmail.com'
MSN_PORT = 1863

CONFIG_PATH = path.join(getenv('HOME'), '.omsn/')
CONFIG_FILE = path.join(CONFIG_PATH, 'omsnrc')
DP_CACHE = path.join(CONFIG_PATH, 'dp_cache/') # Display picture cache directory
OWN_DP_FILE = path.join(CONFIG_PATH, 'avatar')
DEF_OWN_DP_FILE = 'images/others/def_avatar.png'

ONLINE = papyon.Presence.ONLINE
BUSY = papyon.Presence.BUSY
IDLE = papyon.Presence.IDLE
AWAY = papyon.Presence.AWAY
BE_RIGHT_BACK = papyon.Presence.BE_RIGHT_BACK
ON_THE_PHONE = papyon.Presence.ON_THE_PHONE
OUT_TO_LUNCH = papyon.Presence.OUT_TO_LUNCH
OFFLINE = papyon.Presence.OFFLINE
INVISIBLE = papyon.Presence.INVISIBLE

MAX_NICK_LEN = 25

STATUS_LIST = \
(
    ONLINE,
    BUSY,
    IDLE,
    AWAY,
    BE_RIGHT_BACK,
    ON_THE_PHONE,
    OUT_TO_LUNCH,
    OFFLINE,
    INVISIBLE
)

STATUS_NAMES = \
{
    ONLINE : 'En ligne',
    BUSY : 'Occupé(e)',
    IDLE : 'Inactif',
    AWAY : 'Absent(e)',
    BE_RIGHT_BACK : 'Bientôt de retour',
    ON_THE_PHONE : 'Au téléphone',
    OUT_TO_LUNCH : 'Parti(e) manger',
    INVISIBLE : 'Invisible',
    OFFLINE : 'Hors ligne'
}

STATUS_PICS_PATH = 'images/status/'

STATUS_PIC_NAMES = \
{
    ONLINE : 'online.png',
    BUSY : 'busy.png',
    IDLE : 'away.png',
    AWAY : 'away.png',
    BE_RIGHT_BACK : 'be_right_back.png',
    ON_THE_PHONE : 'busy.png',
    OUT_TO_LUNCH : 'away.png',
    INVISIBLE : 'offline.png',
    OFFLINE : 'offline.png'
}

STATUS_PIC = {}

class Config(gobject.GObject):

    __gsignals__ = \
    {
        # emitted when connexion AND the opening of the main window is complete
        'connected' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ())
    }

    def __init__(self):

        ''' Load global config '''

        gobject.GObject.__init__(self)
        # Init tree
        try:
            mkdir(CONFIG_PATH)
            mkdir(DP_CACHE)
        except:
            pass

        # If the display picture does not exist, we copy the default one
        if not path.exists(OWN_DP_FILE):
            copy(DEF_OWN_DP_FILE, OWN_DP_FILE)

        # Global objects
        self.already_accepted = [] # see contact_list.py
        self.client = None
        self.chats = None
        self.profile = None
        self.clist = None
        self.clistframe = None
        self.contact_events = None
        self.addressbook_events = None
        self.buttons = None
        self.main_win = None
        self.invite_events = None

        # Parse config if it exists
        self.cfg = {}
        if not path.exists(CONFIG_FILE):
            print 'file does not exists'
            self.cfg['login'] = ''
            self.cfg['passwd'] = ''
            self.cfg['nickname'] = ''
        else:
            cfg_parser = ConfigParser()
            cfg_parser.read(CONFIG_FILE)
            self.cfg['login'] = cfg_parser.defaults()['login']
            self.cfg['passwd'] = cfg_parser.defaults()['passwd']
            self.cfg['nickname'] = cfg_parser.defaults()['nickname']

    def init_main_objects(self):

        ''' Initialize main objects, link them with signals
            and returns all the items we need to display on the
            main window '''

        print "INIT MAIN OBJECTS"

        from chats import Chats
        from profile import Profile
        from contact_list import ContactList
        from gui.contact_list import ContactListFrame
        from contact_events import ContactEvents
        from addressbook_events import AddressBookEvents
        from gui.general_buttons import GeneralButtons
        from content_roaming import CRoaming
        from gl_fct import quit_omsn
        from invite_events import InviteEvents

        # Main objects
        client = self.client
        self.chats = chats = Chats(client)
        self.notebooks = notebooks = chats.notebooks
        self.profile = profile = Profile(client, self.cfg['nickname'])
        self.addressbook_events = AddressBookEvents(client)

        self.clist = clist = ContactList(client)
        self.addressbook_events.clist = clist
        self.clistframe = clistframe = ContactListFrame(clist)
        self.contact_events = ContactEvents(self.client, clist)

        self.buttons = buttons = GeneralButtons()
        self.cr = CRoaming(client._sso, client.address_book)
        self.invite_events = InviteEvents(self.client)

        # Links objects
        clistframe.treeview.connect('cursor-changed', chats.engage_conv, None)
        buttons.add_contact_win.connect('add_contact', clist.add_contact)
        buttons.rm_contact_win.connect('rm_contact', clist.rm_contact)
        buttons.block_contact_win.connect('block_contact', clist.block_contact)
        buttons.block_contact_win.connect('unblock_contact', \
                clist.unblock_contact)
        buttons.connect('quit', quit_omsn)

        # Returns all the objets we need for the main window
        return (notebooks, profile, clistframe, buttons)

    def __save_user_profile(self, login='', passwd='', nickname=''):

        ''' Write user config file. If values are not precised,
            values are unchanged. '''

        try:
            cfg_f = open(CONFIG_FILE, 'w')
        except:
            print('Error: Unable to open config file for writing')
            sysexit(-1)

        _login = self.cfg['login'] if not login else login
        _passwd = self.cfg['passwd'] if not passwd else passwd
        _nickname = self.cfg['nickname'] if not nickname else nickname

        cfg_f.write('#\n# Config file for oMSN\n#\n\n[DEFAULT]\n\n')
        cfg_f.write('login = %s\n' % _login)
        cfg_f.write('passwd = %s\n' % _passwd)
        cfg_f.write('nickname = %s\n' % _nickname)
        cfg_f.close()

    def login():
        def fget(self):
            return self.cfg['login']
        def fset(self, value):
            self.cfg['login'] = value
            self.__save_user_profile(login=value)
        return locals()
    login = property(**login())

    def passwd():
        def fget(self):
            return self.cfg['passwd']
        def fset(self, value):
            self.cfg['passwd'] = value
            self.__save_user_profile(passwd=value)
        return locals()
    passwd = property(**passwd())

    def nickname():
        def fget(self):
            return self.cfg['nickname']
        def fset(self, value):
            self.cfg['nickname'] = value
            self.__save_user_profile(nickname=value)
        return locals()
    nickname = property(**nickname())

gl_config = Config()
