# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import gtk
import gobject

from gui.smileys_widget import SmileysWidget
from gui.embedded_buffer import EmbeddedBuffer

class TextZone(gtk.TextView):

    __gsignals__ = \
    {
        'enter_pressed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            ())
    }

    def _key_pressed(self, widget, event):
        if not event.keyval in [gtk.keysyms.KP_Enter, gtk.keysyms.Return]:
            return False
        self.emit('enter_pressed')
        return True

    def __init__(self):
        gtk.TextView.__init__(self, buffer=EmbeddedBuffer())
        self.set_wrap_mode(gtk.WRAP_WORD_CHAR)
        self.set_size_request(-1, 60)
        self.connect('key-press-event', self._key_pressed)

    @property
    def text(self):
        return self.buffer.props.text

class ChatEntry(gtk.VBox):

    __gsignals__ = \
    {
        'msg_sent' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
            (gobject.TYPE_STRING,))
    }

    def send_msg_sent_sig(self, widget):

        self.emit('msg_sent', self.buffer.get_raw_text())
        self.buffer.set_text('')
        self.textzone.grab_focus()

    def insert_smiley(self, widget, smiley):

        self.textzone.grab_focus()
        self.buffer.insert_at_cursor(' %s ' % smiley)

    def grab_focus(self):
        self.textzone.grab_focus()

    def __init__(self):

        self.textzone = TextZone()
        self.buffer = self.textzone.get_buffer()
        self.textzone.connect('enter_pressed', self.send_msg_sent_sig)
        textzoneframe = gtk.Frame()
        textzoneframe.add(self.textzone)

        bsend = gtk.Button('Envoyer')
        bsend.connect('clicked', self.send_msg_sent_sig)

        smiley_w = SmileysWidget()
        smiley_w.connect('insert_smiley', self.insert_smiley)

        hbox = gtk.HBox(spacing=5)
        hbox.pack_start(textzoneframe, expand=True, fill=True)
        hbox.pack_start(smiley_w, expand=False)
        hbox.pack_start(bsend, expand=False)

        gtk.VBox.__init__(self, homogeneous=False, spacing=3)
        lbl = gtk.Label('Envoyer un message :')
        lbl.set_alignment(0, 0.5)
        self.pack_start(lbl, expand=False)
        self.pack_start(hbox, expand=True, fill=True)
