# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import gtk
import gobject

from smileys import SMILEYS_NAME, SMILEYS_PIXBUF
from gui.chat_entry import ChatEntry

import time

RELATED_COLORS = \
[
    '#FFC5A9', # Incoming messages
    '#D4E0FF', # Outgoing messages
    '#808080', # Incoming offline messages
    '#90949D', # Outgoing offline messages
    '#A1E0A7', # Information
    '#FF4646'  # Error/Warning
]

class ChatTagConv(gtk.TextTag):

    ''' For embedded bloc text in chat "window" '''

    def __init__(self, color):

        gtk.TextTag.__init__(self)
        self.set_property('paragraph-background', color)
        self.set_property('left-margin', 10)
        self.set_property('right-margin', 10)
        self.set_property('pixels-above-lines', 10)
        self.set_property('pixels-below-lines', 10)
        self.set_property('indent', 10)

class ChatTextView(gtk.TextView):

    ''' Improved TextView widget for conversation '''

    def __get_smileys_ref(self, text):

        ''' Make and return an index of the smileys in the sent text '''

        smileys_ref = []
        for smiley in SMILEYS_NAME:
            pos = text.find(smiley)
            while pos != -1:
                smileys_ref.append((pos, smiley))
                pos = text.find(smiley, pos + 1)
        smileys_ref.sort()
        return smileys_ref

    def __insert_text_bloc(self, position, text):

        ''' Insert with smileys '''

        smileys_ref = self.__get_smileys_ref(text)
        text_focus = 0
        for (sm_pos, sm_str) in smileys_ref:
            if text_focus < sm_pos:
                self.buffer.insert(self.buffer.get_end_iter(), text[text_focus:sm_pos])
            self.buffer.insert_pixbuf(self.buffer.get_end_iter(), SMILEYS_PIXBUF[sm_str])
            text_focus = sm_pos + len(sm_str)
        self.buffer.insert(self.buffer.get_end_iter(), text[text_focus:])

    def add_entry_content(self, io_type, msg, who=''):

        ''' Write the embedded text in the TextBuffer '''

        # Head (Name, time)
        self.buffer.insert(self.buffer.get_end_iter(), '\n')
        if who != '':
            the_time = time.strftime('%Hh%M')
            head = '%s à %s\n' % (who, the_time)
            self.buffer.insert(self.buffer.get_end_iter(), head)

        # Text bloc
        begin_text = self.buffer.get_end_iter()
        begin_mark = self.buffer.create_mark('begin_text', begin_text, left_gravity=True)
        self.__insert_text_bloc(begin_text, msg + '\n')
        begin_text = self.buffer.get_iter_at_mark(begin_mark)
        end_text = self.buffer.get_end_iter()
        self.buffer.apply_tag(self.tag_conv[io_type], begin_text, end_text)

        # Focus on last lines
        focus = self.buffer.get_end_iter()
        mark = self.buffer.create_mark('end', focus, False)
        self.scroll_to_mark(mark, 0.05, True, 0.0, 1.0)

    def __init__(self):

        self.i = 0
        gtk.TextView.__init__(self)

        self.set_wrap_mode(gtk.WRAP_WORD_CHAR)
        self.set_editable(False)
        self.set_cursor_visible(False)

        # tag_conv are related to incoming/outcoming messages
        self.tag_conv = [ChatTagConv(c) for c in RELATED_COLORS]
        self.buffer = self.get_buffer()
        for tag in self.tag_conv:
            self.buffer.get_tag_table().add(tag)

class ChatView(gtk.VBox):

    ''' ChatText with scroll, user entry and send button '''

    def __grab_entry_focus(self):
        self.entry.grab_focus()
        return False

    def grab_entry_focus(self):
        gobject.timeout_add(500, self.__grab_entry_focus)

    def add_entry_content(self, *params):
        self.chattextview.add_entry_content(*params)

    def __init__(self):

        gtk.VBox.__init__(self, spacing=5)

        self.chattextview = ChatTextView()
        self.entry = ChatEntry()

        # ScrolledWindow with frame
        swin = gtk.ScrolledWindow()
        swin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        swin.add(self.chattextview)
        frame = gtk.Frame()
        frame.add(swin)

        # Packaging
        self.pack_start(frame, expand=True, fill=True)
        self.pack_start(self.entry, expand=False)
