# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import gtk
import papyon

from gl_config import gl_config
from gui.contact_list import ContactListFrame

class CListActionWin(gtk.Window):

    def get_selected_contact(self):

        treeviewmodel = self.treeview.get_model()
        (path, column_focus) = self.treeview.get_cursor()
        if not path:
            return None
        selected_iter = treeviewmodel.get_iter(path)
        contact = treeviewmodel.get_value(selected_iter, 0)
        if isinstance(contact, papyon.Contact):
            return contact
        return None

    def update_button(self, widget, user_param=None):

        contact = self.get_selected_contact()
        self.button.set_sensitive(isinstance(contact, papyon.Contact))

    def __init__(self, button, text='', title=''):

        gtk.Window.__init__(self)
        self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
        self.set_title(title)
        self.set_border_width(10)
        self.set_size_request(600, 500)
        self.set_resizable(False)
        self.set_modal(True)

        # Items
        self.button = button

        info_img = gtk.Image()
        info_img.set_from_stock(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_DIALOG)

        lbl = gtk.Label(text)
        lbl.set_line_wrap(True)

        clistframe = ContactListFrame(gl_config.clist)
        self.treeview = clistframe.treeview
        self.treeview.connect('cursor-changed', self.update_button, None)

        close_b = gtk.Button(stock=gtk.STOCK_CLOSE)
        close_b.connect('clicked', lambda w: self.hide())
        self.set_decorated(False)

        info_box = gtk.HBox(spacing=3, homogeneous=False)
        info_box.pack_start(info_img, expand=False)
        info_box.pack_start(lbl, expand=True, fill=True)

        buttons_box = gtk.HBox(spacing=3)
        buttons_box.pack_start(close_b)
        if button:
            buttons_box.pack_start(button)
            button.set_sensitive(False)

        box = gtk.VBox(homogeneous=False, spacing=5)
        box.pack_start(info_box, expand=False)
        box.pack_start(clistframe, expand=True, fill=True)
        box.pack_start(buttons_box, expand=False)

        self.add(box)
