# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import papyon
import gtk

from gui.action_button import ActionButton
from gui.dp_widget import DPWidget

class ContactBoxInfo(gtk.HBox):

    ''' Item of the InfoBox '''

    def update_avatar(self):
        self.display_pic.update_dp(self._contact.dp)

    def update_display_name(self):
        self.lbl_presence.set_label(self._contact.display_name + self.presence_str[self._contact.presence])
        if (self._contact.client_capabilities.has_webcam):
            self.lbl_webcam.set_label('Cliquer pour démarrer une conversation avec webcam')
        else:
            self.lbl_webcam.set_label(self._contact.display_name + ' n\'a pas de webcam')

    def update_display_presence(self):
        self.lbl_presence.set_label(self._contact.display_name + self.presence_str[self._contact.presence])

    def update_warning_label(self):
        if self._contact.memberships & papyon.Membership.BLOCK:
            self.lbl_warning.set_markup('<span foreground="red"><u>%s</u></span>' % 'Attention, vous avez bloqué ce contact et il ne peut donc pas vous répondre')
        else:
            self.lbl_warning.set_label('')

    def update_display_personal_message_changed(self):
        if (self._contact.personal_message != ''):
            self.lbl_perso_msg = gtk.Label('Message personnel: ' + self._contact.personal_message)
        else:
            self.lbl_perso_msg = gtk.Label('')

    def __init__(self, contact):
        self.presence_str = {}
        self.presence_str['NLN'] = ' est présent devant son ordinateur.'
        self.presence_str['BSY'] = ' est occupé(e).'
        self.presence_str['IDL'] = ' est inactif.'
        self.presence_str['AWY'] = ' est abscent.'
        self.presence_str['BRB'] = ' est bienttôt de retour.'
        self.presence_str['PHN'] = ' est au téléphone.'
        self.presence_str['LUN'] = ' est en train de manger.'
        self.presence_str['HDN'] = ' est contact est invisible.'
        self.presence_str['FLN'] = ' est hors ligne.'

        self._contact = contact

        ''' Ajout d'information sur le contact'''

        if (self._contact.personal_message != ''):
            self.lbl_perso_msg = gtk.Label('Message personnel: ' + self._contact.personal_message)
        else:
            self.lbl_perso_msg = gtk.Label('')
        self.lbl_presence = gtk.Label(self._contact.display_name + self.presence_str[self._contact.presence])
        self.display_pic = DPWidget(contact.dp)
        self.lbl_warning = gtk.Label()
        self.update_warning_label()

        self.lbl_warning.set_line_wrap(True)
        self.lbl_warning.set_justify(gtk.JUSTIFY_LEFT)

        self.lbl_perso_msg.set_line_wrap(True)
        self.lbl_presence.set_line_wrap(True)
        self.lbl_perso_msg.set_justify(gtk.JUSTIFY_LEFT)
        self.lbl_presence.set_justify(gtk.JUSTIFY_LEFT)

        vbox1 = gtk.VBox(spacing=3)
        vbox2 = gtk.VBox(spacing=3)
        hbox = gtk.HBox(spacing=3)

        vbox1.pack_start(self.display_pic)
        vbox2.pack_start(self.lbl_presence)
        vbox2.pack_start(self.lbl_perso_msg)
        vbox2.pack_start(self.lbl_warning)

        if (self._contact.client_capabilities.has_webcam):
            hbox_webcam = gtk.HBox()
            b_webcam = gtk.Button()
            self.lbl_webcam = gtk.Label()
            self.img_webcam = gtk.Image()
            self.img_webcam.set_from_file('images/others/webcam.png')
            self.lbl_webcam.set_label('Cliquer pour démarrer une conversation avec webcam')
            hbox_webcam.pack_start(self.lbl_webcam)
            hbox_webcam.pack_start(self.img_webcam)
            b_webcam.add(hbox_webcam)
            vbox2.pack_start(b_webcam)

        hbox.pack_start(vbox1)
        hbox.pack_start(vbox2)
        gtk.HBox.__init__(self)
        self.set_border_width(5)
        self.pack_start(hbox, expand=False)

class InfoBox(gtk.ScrolledWindow):

    ''' Pack all ContactBoxInfo (for each contact) '''

    def get_participant_box(self, participant):
        if participant in self.participants:
            return self.participants[participant]
        else:
            return None

    def remove_participant(self, participant):
        if len(self.participants) > 1 and participant in self.participants:
            box = self.participants[participant]
            box.destroy()
            del(box)

    def new_participant(self, participant):
        print(' |--- New participant [%s]' % participant.account)
        if not participant in self.participants:
            box = ContactBoxInfo(participant)
            box.show_all()
            self.main_box.pack_start(box)
            self.participants[participant] = box

    def __init__(self, participants):

        self.participants = {}

        self.main_box = gtk.HBox()
        for participant in participants:
            self.new_participant(participant)

        gtk.ScrolledWindow.__init__(self)
        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
        self.add_with_viewport(self.main_box)
