# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import gtk
import webbrowser
from time import sleep

from gl_config import gl_config, WIN_TITLE, OWN_DP_FILE
from gl_fct import quit_omsn

from gui.dp_widget import DPWidget
from gui.action_button import ActionButton

class StatusInfo(gtk.Alignment):

    def __init__(self):
        self.img = gtk.Image()
        self.img.set_from_file('images/others/loading_big.gif')
        self.img.set_no_show_all(True)
        self.label = gtk.Label()
        box = gtk.VBox(homogeneous=False, spacing=10)
        box.pack_start(self.img, expand=False)
        box.pack_start(self.label, expand=False)
        gtk.Alignment.__init__(self, .5, .5)
        self.add(box)

    def set_text(self, text):
        if text == '':
            self.img.hide()
        else:
            self.img.show()
        self.label.set_text(text)

class LoginWindow(gtk.Window):

    def __switch_buttons_sensitive(self, value):

        ''' Set all buttons unclickable '''

        self.login_tb.set_sensitive(value)
        self.passwd_tb.set_sensitive(value)
        self.register_lb.set_sensitive(value)
        self.reset_pass_lb.set_sensitive(value)

    def update_login_state(self, state, state_str):

        ''' This is called when state changes '''

        self.pgbar.set_fraction(state / self.nb_steps)
        self.status_lbl.set_text(state_str)
        while gtk.events_pending():
            gtk.main_iteration(False)

    def display_error(self, err, anim=False):
        # Reset display
        self.__switch_buttons_sensitive(True)
        self.status_lbl.set_text('')
        self.passwd_tb.grab_focus()
        self.pgbar.set_fraction(0)
        self.pgbar.hide()
        self.cancel_b.hide()
        self.connect_b.show()
        self.connect_b.grab_default()

        err = '<span foreground="#FF0000"><b>%s</b></span>' % err
        if anim:
            for i in range(10):
                self.err_lbl.set_use_markup(True)
                str = ('\t%s' if i % 2 else '%s\t') % err
                self.err_lbl.set_markup(str)
                while gtk.events_pending():
                    gtk.main_iteration(False)
                sleep(.05)
        self.err_lbl.set_markup(err)

    def cancel_connexion(self):
        pass

    def login_display_mod(self, nb_steps):

        ''' Switch to login procedure '''

        self.__switch_buttons_sensitive(False)
        self.nb_steps = nb_steps * 1.
        self.connect_b.hide()
        self.cancel_b.show()
        self.err_lbl.set_text('')
        self.pgbar.show()

    def __init__(self):

        gtk.Window.__init__(self)
        self.connect('delete_event', quit_omsn)
        self.set_title(WIN_TITLE)
        self.set_default_size(800, 600)

        # Display picture
        display_pic = DPWidget(OWN_DP_FILE)

        # Label error
        self.err_lbl = gtk.Label()
        self.__err_status = 0
        self.__err_active = False

        # Connexion info
        self.status_lbl = StatusInfo()

        # Login label
        login_lbl = gtk.Label('Identifiant : ')

        # Login text box
        self.login_tb = gtk.Entry()
        self.login_tb.set_activates_default(True)

        # Login container
        login_hbox = gtk.HBox(homogeneous=False)
        login_hbox.pack_end(self.login_tb, expand=False)
        login_hbox.pack_end(login_lbl, expand=False)

        # Password label
        passwd_lbl = gtk.Label('Mot de passe : ')

        # Password text box
        self.passwd_tb = gtk.Entry()
        self.passwd_tb.set_visibility(False)
        self.passwd_tb.set_activates_default(True)

        # Password container
        passwd_hbox = gtk.HBox(homogeneous=False)
        passwd_hbox.pack_end(self.passwd_tb, expand=False)
        passwd_hbox.pack_end(passwd_lbl, expand=False)

        # Connect button
        self.connect_b = ActionButton('Connexion à MSN', gtk.STOCK_CONNECT)
        self.connect_b.set_property('can-default', True)

        # Cancel button
        self.cancel_b = gtk.Button(stock=gtk.STOCK_CANCEL)
        self.cancel_b.connect('clicked', self.cancel_connexion)

        # Progress login bar : remplace the button
        self.pgbar = gtk.ProgressBar()

        # Button (and progressbar) container
        self.connect_b_container = gtk.HBox()
        self.connect_b_container.pack_start(self.pgbar, expand=True, fill=False)
        self.connect_b_container.pack_start(self.connect_b, expand=True, fill=False)
        self.connect_b_container.pack_start(self.cancel_b, expand=True, fill=False)

        # Pack entries and associated labels + connect button
        container = gtk.VBox()
        container.set_border_width(20)
        container.pack_start(login_hbox, expand=False, padding=5)
        container.pack_start(passwd_hbox, expand=False, padding=5)
        container.pack_start(self.connect_b_container, expand=False, padding=5)

        # Big container (with a frame) to center the container above
        frame = gtk.Frame()
        frame.add(container)
        big_container = gtk.Alignment(0.5, 0.5)
        big_container.add(frame)

        # Extras links
        self.register_lb = gtk.LinkButton('', label='Inscription à MSN')
        self.register_lb.set_uri('https://accountservices.passport.net/reg.srf?sl=1&lc=1033')
        self.reset_pass_lb = gtk.LinkButton('', label='J\'ai oublié mon mot de passe')
        self.reset_pass_lb.set_uri('https://accountservices.passport.net/uiresetpw.srf?lc=1033')
        extras_links_box = gtk.VBox()
        extras_links_box.pack_start(self.register_lb)
        extras_links_box.pack_start(self.reset_pass_lb)
        gtk.link_button_set_uri_hook(lambda w,u: webbrowser.open(u))

        # Extras links container (for vertical alignement)
        extras_links_box_align = gtk.HBox(homogeneous=False)
        extras_links_box_align.pack_start(extras_links_box, expand=True, fill=False)

        # Main container (contains everything in the window)
        main_container = gtk.VBox()
        main_container.pack_start(display_pic, expand=False, padding=50)
        main_container.pack_start(self.err_lbl, expand=False)
        main_container.pack_start(self.status_lbl, expand=False)
        main_container.pack_start(big_container, expand=False, padding=50)
        main_container.pack_start(extras_links_box_align, expand=False, padding=0)

        # Final pack and display
        self.add(main_container)
        self.show_all()
        self.pgbar.hide()
        self.cancel_b.hide()

        # Focus, misc
        self.connect_b.grab_default()
        if gl_config.login:
            self.login_tb.set_text(gl_config.login)
            if gl_config.passwd:
                self.passwd_tb.set_text(gl_config.passwd)
            self.passwd_tb.grab_focus()
        else:
            self.login_tb.grab_focus()

    @property
    def login(self):
        return self.login_tb.get_text()

    @property
    def passwd(self):
        return self.passwd_tb.get_text()
