# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import gtk
import gobject

from gl_config import gl_config
from gl_fct import format_nick

class NickEntry(gtk.Entry):

    ''' Entry widget for NickChanger '''

    def _key_pressed(self, widget, event):
        if event.keyval in [gtk.keysyms.KP_Enter, gtk.keysyms.Return]:
            self.emit('focus-out-event', None)
            return True
        if event.keyval == gtk.keysyms.Escape:
            self.set_text(gl_config.nickname)
            self.emit('focus-out-event', None)
            return True
        return False

    def __init__(self, nick):
        gtk.Entry.__init__(self)
        self.set_text(nick)
        self.connect('key-press-event', self._key_pressed)

class ValidEntryButton(gtk.Button):

    ''' Just a tiny button to valid the entry '''

    def __init__(self):
        img = gtk.Image()
        img.set_from_stock(gtk.STOCK_APPLY, gtk.ICON_SIZE_BUTTON)
        gtk.Button.__init__(self)
        self.set_image(img)

class NickButton(gtk.Button):

    ''' Button widget for NickChanger '''

    def __init__(self, nick):
        gtk.Button.__init__(self, nick)
        self.set_relief(gtk.RELIEF_NONE)

class EntryAndButon(gtk.HBox):

    ''' Just a pack for the entry + the little button '''

    def __init__(self, entry, button):
        gtk.HBox.__init__(self, homogeneous=False)
        self.pack_start(entry, expand=True, fill=True)
        self.pack_start(button, expand=False)

class NickChanger(gtk.VBox):

    ''' Personnal widget button which simulate a
        transformation to an entry when clicked, and
        get back to button when loosing focus '''

    __gsignals__ = \
    {
        'nickname_changed' : \
            (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (str,))
    }

    def set_nickname(self, nick):
        self.button.set_label(format_nick(nick))
        self.emit('nickname_changed', nick)

    def switch_widget(self, widget=None, event=None):

        if self.current_widget == self.entry:
            self.entry_and_button.hide()
            self.button.show()
            new_nick = self.entry.get_text()
            if new_nick != gl_config.nickname:
                self.set_nickname(new_nick)
            self.current_widget = self.button
        else:
            self.button.hide()
            self.entry_and_button.show_all()
            self.current_widget = self.entry
            self.entry.grab_focus()

    def __init__(self, nick):

        self.nick = nick

        self.entry = NickEntry(nick)
        self.button = NickButton(nick)
        self.vbutton = ValidEntryButton()
        self.entry_and_button = EntryAndButon(self.entry, self.vbutton)

        self.button.connect('clicked', self.switch_widget)
        self.entry.connect('focus-out-event', self.switch_widget)
        self.vbutton.connect('clicked', self.switch_widget)

        self.current_widget = self.button
        self.current_widget.show()

        gtk.VBox.__init__(self)
        self.pack_start(self.entry_and_button)
        self.pack_start(self.button)
        self.show()
