# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import gtk
import gobject

from gui.chatnotebook import ChatNotebook, NotebookLabel
from chat_instance import ChatInstance

class InfoFrame(gtk.Frame):

    ''' When no conversations are engaged, this frame is displayed '''

    INFO_TITLE = 'Information'
    INFO_CONTENT = 'Cliquez sur un des contacts à votre gauche ' \
        'pour engager une conversation.'

    def __init__(self):

        gtk.Frame.__init__(self)
        self.set_border_width(5)

        # Info icon
        info_icon = gtk.Image()
        info_icon.set_from_stock(gtk.STOCK_DIALOG_INFO, gtk.ICON_SIZE_DIALOG)

        # Info text
        lbl_content = gtk.Label(InfoFrame.INFO_CONTENT)

        # Packaging
        box = gtk.HBox()
        box.pack_start(info_icon)
        box.pack_start(lbl_content)
        self.add(box)

class Notebooks(gtk.Notebook):

    ''' All the notebooks '''

    def __disp_init_info_frame(self):

        ''' Display the info frame because there is no conversation tab '''

        lbl_title = gtk.Label(InfoFrame.INFO_TITLE)
        info_frame = InfoFrame()
        info_frame.show_all()
        self.append_page(info_frame, lbl_title)
        self.display_info = 1

    def close_tab(self, page_num):

        ''' This method should not be called before closing conversation. '''

        self.remove_page(page_num)
        if self.get_n_pages() == 0:
            self.__disp_init_info_frame()

    def _switch_page(self, notebook, page, page_num):

        chat_instance = self.get_nth_page(page_num)
        if not isinstance(chat_instance, ChatInstance):
            return
        chat_instance.unset_highlight_nb()
        chat_instance.chatview.grab_entry_focus()

    def append_chat(self, chatinstance):

        new_tab = self.append_page(chatinstance, chatinstance.tablbl)
        if self.display_info == 1:
            self.remove_page(0)
            self.display_info = 0
        return new_tab

    def __init__(self):

        gtk.Notebook.__init__(self)
        self.set_scrollable(True)
        self.set_border_width(5)
        self.__disp_init_info_frame()
        self.connect('switch-page', self._switch_page)

