# -*- coding: utf-8 -*-

##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import gtk
import gobject

from subprocess import Popen, PIPE
from os import path, getenv

from gl_config import ONLINE, OWN_DP_FILE

from gui.nick_changer import NickChanger
from gui.dp_widget import DPWidget
from gui.status_list_widget import StatusListWidget

class ProfileDP(DPWidget):

    __gsignals__ = \
    {
        'dp_changed' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (str,))
    }

    def select_new_dp(self, widget=None):

        ''' Test if Ordissimo file selector exists, else use GTK '''

        img_path = path.join(getenv('HOME'), 'Images')
        ocmd = '/usr/bin/navigation-ouvrir'
        ocmd_p = '-d %s -p image/jpeg image/png image/gif' % img_path
        args = ('%s %s' % (ocmd, ocmd_p)).split()
        if path.exists(ocmd):
            process = Popen(args=args, stdout=PIPE)
            process.wait()
            filename = process.stdout.readline().strip()
            if filename:
                self.emit('dp_changed', filename)
                self.update_dp(filename)
            return

        t = 'Sélectionnez une image'
        a = gtk.FILE_CHOOSER_ACTION_OPEN
        b = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK)
        file_chooser = gtk.FileChooserDialog(title=t, action=a, buttons=b)
        res = file_chooser.run()
        if res == gtk.RESPONSE_OK:
            filename = file_chooser.get_filename()
            self.emit('dp_changed', filename)
            self.update_dp(filename)
        file_chooser.destroy()

    def __init__(self, pic):
        DPWidget.__init__(self, pic)
        self.dp_button.connect('clicked', self.select_new_dp)

class ProfileBox(gtk.Frame):

    __gsignals__ = \
    {
        'nickname_changed' : \
            (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (str,)),
        'dp_changed' : \
            (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (str,)),
        'status_changed' : \
            (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (type(ONLINE),))
    }

    def __init__(self, nickname):

        # Display picture
        self.dp_w = ProfileDP(OWN_DP_FILE)
        self.dp_w.connect('dp_changed',
            lambda w, s: self.emit('dp_changed', s))
        self.dp_w.show_all()

        # Nickname
        self.nick_w = NickChanger(nickname)
        self.nick_w.connect('nickname_changed', \
            lambda w, s: self.emit('nickname_changed', s))
        self.nick_w.show()

        # Status list switcher
        status_w = StatusListWidget()
        status_w.connect('status_changed',
            lambda w, s: self.emit('status_changed', s))
        status_w.show_all()

        # Packaging
        box = gtk.VBox(spacing=3)
        box.pack_start(self.dp_w)
        box.pack_start(self.nick_w)
        box.pack_start(status_w)
        box.set_border_width(5)
        box.show()

        gtk.Frame.__init__(self, 'Mon profil')
        self.add(box)
