#! /usr/bin/env python
# -*- coding: utf-8 -*-

##
##        oMSN
##    Ordissimo MSN
##
#########################
##
##    Copyright © 2008-2009 Ordissimo <info@substantiel.fr>
##    Copyright © 2008-2009 Boesch Clément <ubitux@gmail.com>
##
##    This file is part of oMSN.
##
##    oMSN is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    oMSN is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with oMSN.  If not, see <http://www.gnu.org/licenses/>.
##

import papyon
import contact_extend

from gl_config import gl_config, MSN_SERVER, MSN_PORT

from client import ClientEvents

import pygtk
pygtk.require('2.0')
import gtk

from gui.login import LoginWindow
from gui.main_window import MainWindow

STATE_STR = \
{
    papyon.event.ClientState.CLOSED : 'Connexion closed',
    papyon.event.ClientState.CONNECTING : 'Connexion au serveur MSN…',
    papyon.event.ClientState.CONNECTED : 'Connexion réussie',
    papyon.event.ClientState.AUTHENTICATING : 'Identification en cours…',
    papyon.event.ClientState.AUTHENTICATED : 'Mot de passe accepté',
    papyon.event.ClientState.SYNCHRONIZING : 'Récupération de la liste des contacts',
    papyon.event.ClientState.SYNCHRONIZED : 'Liste de contacts récupérée, bon chat !',
    papyon.event.ClientState.OPEN : 'Session ouverte'
}

ERR_TYPE_STR = \
{
    papyon.event.ClientErrorType.NETWORK : \
        'Erreur de réseau. Les serveurs Microsoft sont indisponibles.',
    papyon.event.ClientErrorType.AUTHENTICATION : \
        'Erreur d\'authentification. Vérifiez votre identifiant ' \
        'et votre mot de passe',
    papyon.event.ClientErrorType.PROTOCOL : \
        'Erreur : conflit. Votre compte semble être connecté depuis un autre endroit.',
    papyon.event.ClientErrorType.ADDRESSBOOK : \
        'Erreur de récuperation de la liste des contacts sur les ' \
        'serveurs Microsoft'
}

class Login(LoginWindow):

    def client_state_changed(self, state):

        ''' Called login state changed '''

        if state not in STATE_STR:
            print('Unknown connection state: %d' % state)
        elif self.old_state != state:
            if state == papyon.event.ClientState.CLOSED:
                self.update_login_state(0, '')
            else:
                self.update_login_state(state, STATE_STR[state])
            print(' |--- ' + STATE_STR[state])
        self.old_state = state

        # Check if we are connected
        if state == papyon.event.ClientState.OPEN:

            # Update config. Note that the config is writen on any changement
            if not gl_config.nickname:
                gl_config.nickname = self.login
            gl_config.login = self.login
            gl_config.passwd = self.passwd

            # Open session
            profile = gl_config.client.profile
            profile.presence = papyon.Presence.ONLINE
            profile.display_name = gl_config.nickname

            # Load main window
            self.pgbar.hide()
            # because we don't want it if the
            # window is redisplayed on disconnection

            self.hide()
            main_objects = gl_config.init_main_objects()
            self.main_win = MainWindow(*main_objects)
            self.main_win.show()

            return True
        elif state == papyon.event.ClientState.CLOSED:
            if self.main_win:
                self.main_win.hide()
                self.show()
        return False

    def client_error(self, error_type):

        ''' Error during login '''

        error_type = int(error_type)
        if not error_type in ERR_TYPE_STR:
            err = 'Erreur de type inconnu...'
        else:
            err = ERR_TYPE_STR[error_type]
        self.display_error(err)
        return False

    def cancel_connexion(self, widget=None):
        try:
            #TODO: need something better…
            client = gl_config.client
            client._transport._transport.close()
            client._address_book = None
            client._oim_box = None
            client._spaces = None
            client._state = papyon.event.ClientState.CLOSED
        finally:
            self.display_error('')

    def __login(self, widget=None):

        ''' Engage login procedure '''

        print('[+] Logging')
        try:
            self.login_display_mod(len(STATE_STR) - 1)
            client = gl_config.client = papyon.Client((MSN_SERVER, MSN_PORT))
            events_h = client._event_handler = ClientEvents(client)
            events_h.connect('state_changed', lambda w, s: self.client_state_changed(s))
            events_h.connect('connect_error', lambda w, e: self.client_error(e))
            client.login(self.login, self.passwd)
        except:
            self.display_error('Erreur de connexion. Veuillez vous reconnecter',
                    anim=True)

    def __init__(self):
        self.old_state = -1
        self.main_win = None
        LoginWindow.__init__(self)
        self.connect_b.connect('clicked', self.__login)

if __name__ == '__main__':
    login = Login()
    gtk.main()
